/*
 * waitdev.c
 *
 * Code to wait for and setup a specific block device.
 *
 * Copyright 2006-2008 Red Hat, Inc.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author(s): Peter Jones <pjones@redhat.com>
 */

#define _GNU_SOURCE 1

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <bdevid.h>

#include <nash.h>
#include "block.h"
#include "blkent.h"
#include "lib.h"
#include "util.h"

void
nashWaitCancel(nashContext *nc)
{
    if (nc->waitdev_iter)
        nashBdevIterEnd(&nc->waitdev_iter);
}

int
nashWaitForDevice(nashContext *nc, char *device, long long wait_usecs)
{
    struct timespec wait;
    int rc = -1;
    int blktab = 0;

    if (device[0] == '@') {
        device += 1;
        blktab = 1;
        nash_load_blktab(nc);
    }

    usectospec(wait_usecs, &wait);

    if (!nc->waitdev_iter) {
        nashBlockInit(nc);

        nc->waitdev_iter = nashBdevIterNewPoll(nc, &wait);
        if (!nc->waitdev_iter)
            return -1;
    }

    do {
        struct nash_block_dev *dev = NULL;
        struct nash_dev_node *node = NULL;
        int ret;

        /* this has a side affect of adding devices to the cache */
        if ((ret = nashBdevIterNext(&nc->waitdev_iter, &dev)) >= 0) {
#if 0
            printf("%s bdev(%s, %s, %d:%d)\n",
                dev->type == ADD ? "add" : "remove",
                dev->sysfs_path, dev->dev_path, major(dev->devno),
                minor(dev->devno));
#endif
            rc = 0;
            if (node || 
                    (node = nash_dev_tree_find_device(nc, device, blktab))) {
                    rc = 1;
                    break;
            }
            if (ret == 0)
                break;
        } else {
            /* timeout */
            break;
        }
    } while (1);

    nashBdevIterEnd(&nc->waitdev_iter);
    nc->waitdev_iter = NULL;
    return rc;
}

/*
 * vim:ts=8:sw=4:sts=4:et
 */
